@@ -1,5 +1,5 @@
 /*
- *	$Id: pciutils.h,v 1.4 1998/03/31 21:02:18 mj Exp $
+ *	$Id: pciutils.h,v 1.8 1998/07/17 08:57:20 mj Exp $
  *
  *	Linux PCI Utilities -- Declarations
  *
@@ -16,7 +16,7 @@
 #include "pci.h"
 #endif
 
-#define PCIUTILS_VERSION "1.04"
+#define PCIUTILS_VERSION "1.07"
 
 #define PROC_BUS_PCI "/proc/bus/pci"
 #define ETC_PCI_IDS "/etc/pci.ids"
@@ -40,6 +40,9 @@ char *lookup_vendor(word);
 char *lookup_device(word, word);
 char *lookup_device_full(word, word);
 char *lookup_class(word);
+char *lookup_subsys_vendor(word);
+char *lookup_subsys_device(word, word);
+char *lookup_subsys_device_full(word, word);
 
 /* filter.c */
 
